@@ -15,6 +15,7 @@
  *   limitations under the License.
  ***************************************************************************/
 
+#include <cstdlib>
 #include <cstdio>
 #include <unistd.h>
 
@@ -30,6 +31,7 @@
 #include <QScreen>
 #include <QString>
 #include <QStringList>
+#include <QSettings>
 #include <QTranslator>
 #include <QLockFile>
 
@@ -42,6 +44,29 @@ void messageHandler(QtMsgType type, const QMessageLogContext &context, const QSt
 
 int main(int argc, char *argv[])
 {
+    if (!getenv("XDG_CURRENT_DESKTOP")) {
+        // The default style in the OOBE environment is hideous and unusable.
+        QApplication::setStyle("windows"); // Qt docs say do this before the QApplication instance.
+        QPalette pal;
+        pal.setColor(QPalette::Window, Qt::black);
+        pal.setColor(QPalette::WindowText, Qt::white);
+        pal.setColor(QPalette::Base, Qt::black);
+        pal.setColor(QPalette::AlternateBase, Qt::black);
+        pal.setColor(QPalette::Text, Qt::white);
+        pal.setColor(QPalette::Button, Qt::black);
+        pal.setColor(QPalette::ButtonText, Qt::white);
+        pal.setColor(QPalette::BrightText, Qt::white);
+        pal.setColor(QPalette::Disabled, QPalette::Text, Qt::darkGray);
+        pal.setColor(QPalette::Disabled, QPalette::WindowText, Qt::darkGray);
+        pal.setColor(QPalette::Disabled, QPalette::ButtonText, Qt::darkGray);
+        pal.setColor(QPalette::Highlight, Qt::lightGray);
+        pal.setColor(QPalette::HighlightedText, Qt::black);
+        pal.setColor(QPalette::ToolTipBase, Qt::black);
+        pal.setColor(QPalette::ToolTipText, Qt::white);
+        pal.setColor(QPalette::Link, Qt::cyan);
+        QApplication::setPalette(pal); // Qt docs say do this after setting the style.
+    }
+
     QApplication a(argc, argv);
     a.setApplicationVersion(VERSION);
     a.setWindowIcon(QIcon("/usr/share/gazelle-installer-data/logo.png"));
@@ -77,19 +102,16 @@ int main(int argc, char *argv[])
             "Without this option, GPT will only be used on drives with at least 2TB capacity.\n"
             "GPT is always used on whole-drive installations on UEFI systems regardless of capacity, even without this option.")},
         {{"m", "mount-keep"}, QObject::tr("Do not unmount /mnt/antiX or close any of the associated LUKS containers when finished.")},
-        {{"n", "nocopy"}, QObject::tr("Another testing mode for installer, partitions/drives are going to be FORMATED, it will skip copying the files.")},
+        {"nocopy", QObject::tr("Another testing mode for installer, partitions/drives are going to be FORMATED, it will skip copying the files.")},
         {{"o", "oem"}, QObject::tr("Install the operating system, delaying prompts for user-specific options until the first reboot.\n"
             "Upon rebooting, the installer will be run with --oobe so that the user can provide these details.\n"
             "This is useful for OEM installations, selling or giving away a computer with an OS pre-loaded on it.")},
         {"oobe", QObject::tr("Out Of the Box Experience option.\n"
             "This will start automatically if installed with --oem option.")},
-        {{"p", "pretend"}, QObject::tr("Test mode for GUI, you can advance to different screens without actially installing.")},
+        {"pretend", QObject::tr("Test mode for GUI, you can advance to different screens without actially installing.")},
         {{"r", "reboot"}, QObject::tr("Reboots automatically when done installing.")},
         {{"s", "sync"}, QObject::tr("Installing with rsync instead of cp on custom partitioning.\n"
-            "-- doesn't format /root and it doesn't work with encryption.")},
-        {"media-check", QObject::tr("Always check the installation media at the beginning.")},
-        {"no-media-check", QObject::tr("Do not check the installation media at the beginning.\n"
-            "Not recommended unless the installation media is guaranteed to be free from errors.")}});
+            "-- doesn't format /root and it doesn't work with encryption.")}});
     parser.addPositionalArgument("config-file", QObject::tr("Load a configuration file as specified by <config-file>."), "<config-file>");
     parser.process(a);
 
@@ -142,19 +164,12 @@ int main(int argc, char *argv[])
 
     // main routine
     qDebug() << "Installer version:" << VERSION;
-    try {
-        MInstall minstall(appConf, parser, cfgfile);
-        const QRect &geo = a.primaryScreen()->availableGeometry();
-        if (parser.isSet("oobe")) minstall.setGeometry(geo);
-        else minstall.move((geo.width() - minstall.width()) / 2, (geo.height() - minstall.height()) / 2);
-        minstall.show();
-        return a.exec();
-    } catch (const char *msg) {
-        qDebug() << "ERROR:" << msg;
-        QMessageBox::critical(nullptr, QString(), QObject::tr(msg));
-        return EXIT_FAILURE;
-    }
-    return EXIT_SUCCESS;
+    MInstall minstall(appConf, parser, cfgfile);
+    const QRect &geo = a.primaryScreen()->availableGeometry();
+    if (parser.isSet("oobe")) minstall.setGeometry(geo);
+    else minstall.move((geo.width() - minstall.width()) / 2, (geo.height() - minstall.height()) / 2);
+    minstall.show();
+    return a.exec();
 }
 
 // The implementation of the handler
