@@ -27,7 +27,6 @@
 #include <QSettings>
 #include <QProcess>
 #include <QProcessEnvironment>
-#include <QDir>
 #include <QFile>
 #include <QFileInfo>
 #include <QTimer>
@@ -37,6 +36,7 @@
 #include <QDebug>
 
 #include "msettings.h"
+#include "checkmd5.h"
 #include "partman.h"
 #include "autopart.h"
 #include "base.h"
@@ -62,13 +62,12 @@ enum Step {
     End
 };
 
-MInstall::MInstall(QSettings &acfg, const QCommandLineParser &args, const QString &cfgfile)
+MInstall::MInstall(QSettings &acfg, const QCommandLineParser &args, const QString &cfgfile) noexcept
     : proc(this), appConf(acfg), appArgs(args), helpBackdrop("/usr/share/gazelle-installer-data/backdrop-textbox.png")
 {
     setupUi(this);
     listLog->addItem("Version " VERSION);
     proc.setupUI(listLog, progInstall);
-    updateCursor(Qt::WaitCursor);
     setWindowFlags(Qt::Window); // for the close, min and max buttons
     textHelp->installEventFilter(this);
     boxInstall->hide();
@@ -82,28 +81,6 @@ MInstall::MInstall(QSettings &acfg, const QCommandLineParser &args, const QStrin
     } else {
         automatic = oem = false;
         pushClose->setText(tr("Shutdown"));
-        phase = 2;
-        // dark palette for the OOBE screen
-        QColor charcoal(56, 56, 56);
-        QPalette pal;
-        pal.setColor(QPalette::Window, charcoal);
-        pal.setColor(QPalette::WindowText, Qt::white);
-        pal.setColor(QPalette::Base, charcoal.darker());
-        pal.setColor(QPalette::AlternateBase, charcoal);
-        pal.setColor(QPalette::Text, Qt::white);
-        pal.setColor(QPalette::Button, charcoal);
-        pal.setColor(QPalette::ButtonText, Qt::white);
-        pal.setColor(QPalette::Active, QPalette::Button, charcoal);
-        pal.setColor(QPalette::Disabled, QPalette::Light, charcoal.darker());
-        pal.setColor(QPalette::Disabled, QPalette::Text, Qt::darkGray);
-        pal.setColor(QPalette::Disabled, QPalette::WindowText, Qt::darkGray);
-        pal.setColor(QPalette::Disabled, QPalette::ButtonText, Qt::darkGray);
-        pal.setColor(QPalette::Highlight, Qt::lightGray);
-        pal.setColor(QPalette::HighlightedText, Qt::black);
-        pal.setColor(QPalette::ToolTipBase, Qt::black);
-        pal.setColor(QPalette::ToolTipText, Qt::white);
-        pal.setColor(QPalette::Link, Qt::cyan);
-        qApp->setPalette(pal);
     }
 
     // setup system variables
@@ -121,12 +98,17 @@ MInstall::MInstall(QSettings &acfg, const QCommandLineParser &args, const QStrin
     // ensure the help widgets are displayed correctly when started
     // Qt will delete the heap-allocated event object when posted
     qApp->postEvent(this, new QEvent(QEvent::PaletteChange));
-    QTimer::singleShot(0, this, [this]() {
+    QTimer::singleShot(0, this, [this]() noexcept {
         try {
             startup();
+            phase = Ready;
         } catch (const char *msg) {
-            proc.log(QStringLiteral("FAILED START - ") + msg, MProcess::Fail);
-            QMessageBox::critical(this, windowTitle(), tr(msg));
+            if(msg[0]) {
+                proc.log(QStringLiteral("FAILED START - ") + msg, MProcess::Fail);
+                QMessageBox::critical(this, windowTitle(), tr(msg));
+            }
+            proc.unhalt();
+            setupAutoMount(true);
             exit(EXIT_FAILURE);
         }
     });
@@ -140,6 +122,7 @@ MInstall::~MInstall() {
     if (partman) delete partman;
     if (autopart) delete autopart;
     if (throbber) delete throbber;
+    if (passCryptoCust) delete passCryptoCust;
 }
 
 // meant to be run after the installer becomes visible
@@ -159,6 +142,26 @@ void MInstall::startup()
             if (ans != QMessageBox::Yes) exit(EXIT_FAILURE);
         }
 
+        // Log live boot command line, looking for the "checkmd5" option.
+        bool nocheck = pretend;
+        QFile fileCLine("/live/config/proc-cmdline");
+        if (fileCLine.open(QFile::ReadOnly | QFile::Text)) {
+            QByteArray data = fileCLine.readAll();
+            nocheck = data.contains("checkmd5\n");
+            proc.log("Live boot: " + data.simplified());
+            fileCLine.close();
+        }
+        // Check the installation media for errors (skip if not required).
+        if (appArgs.isSet("media-check")) nocheck = false;
+        else if (appArgs.isSet("no-media-check")) nocheck = true;
+        if(nocheck) proc.log("No media check");
+        else {
+            checkmd5 = new CheckMD5(proc, labelSplash);
+            checkmd5->check(); // Must be separate from constructor for halt() to work.
+            delete checkmd5;
+            checkmd5 = nullptr;
+        }
+
         partman = new PartMan(proc, *this, appConf, appArgs);
         base = new Base(proc, *partman, *this, appConf, appArgs);
         bootman = new BootMan(proc, *partman, *this, appConf, appArgs);
@@ -166,9 +169,6 @@ void MInstall::startup()
         autopart = new AutoPart(proc, partman, *this, appConf);
         partman->autopart = autopart;
 
-        autoMountEnabled = true; // disable auto mount by force
-        if (!pretend) setupAutoMount(false);
-
         // Link block
         QString link_block;
         appConf.beginGroup("LINKS");
@@ -181,21 +181,14 @@ void MInstall::startup()
         // set some distro-centric text
         textReminders->setPlainText(tr("Support %1\n\n%1 is supported by people like you. Some help others at the support forum - %2, or translate help files into different languages, or make suggestions, write documentation, or help test new software.").arg(PROJECTNAME, PROJECTFORUM)
                             + "\n" + link_block);
+
+        // Password box setup
+        passCryptoCust = new PassEdit(textCryptoPassCust, textCryptoPassCust2, 1, 32, 9, this);
+        connect(passCryptoCust, &PassEdit::validationChanged, pushNext, &QPushButton::setEnabled);
     }
 
-    // Password box setup
-    textCryptoPass->setup(textCryptoPass2, 1, 32, 9);
-    textCryptoPassCust->setup(textCryptoPassCust2, 1, 32, 9);
-    connect(textCryptoPass, &PassEdit::validationChanged, pushNext, &QPushButton::setEnabled);
-    connect(textCryptoPassCust, &PassEdit::validationChanged, pushNext, &QPushButton::setEnabled);
-
     setupkeyboardbutton();
 
-    // Guess if hardware clock is UTC or local time
-    proc.shell("guess-hwclock", nullptr, true);
-    if (proc.readOut() == "localtime") checkLocalClock->setChecked(true);
-    else if (proc.detectMac()) checkLocalClock->setChecked(true);
-
     oobe = new Oobe(proc, *this, this, appConf, oem, modeOOBE);
 
     if (modeOOBE) manageConfig(ConfigLoadB);
@@ -226,27 +219,23 @@ void MInstall::startup()
     }
     oobe->stashServices(true);
 
-    if (modeOOBE) gotoPage(Step::Network);
-    else {
-        textCopyright->setPlainText(tr("%1 is an independent Linux distribution based on Debian Stable.\n\n"
-            "%1 uses some components from MEPIS Linux which are released under an Apache free license."
-            " Some MEPIS components have been modified for %1.\n\nEnjoy using %1").arg(PROJECTNAME));
-        gotoPage(Step::Terms);
-    }
-    updateCursor();
-
-    // automatic installation
-    if (automatic) pushNext->click();
+    textCopyright->setPlainText(tr("%1 is an independent Linux distribution based on Debian Stable.\n\n"
+        "%1 uses some components from MEPIS Linux which are released under an Apache free license."
+        " Some MEPIS components have been modified for %1.\n\nEnjoy using %1").arg(PROJECTNAME));
+    gotoPage(Step::Terms);
 }
 
-void MInstall::splashSetThrobber(bool active)
+void MInstall::splashSetThrobber(bool active) noexcept
 {
     if (active) {
         if (throbber) return;
         labelSplash->installEventFilter(this);
         throbber = new QTimer(this);
-        connect(throbber, &QTimer::timeout, this, &MInstall::splashThrob);
-        throbber->start(100);
+        connect(throbber, &QTimer::timeout, this, [=]() noexcept {
+            ++throbPos;
+            labelSplash->update();
+        });
+        throbber->start(120);
     } else {
         if (!throbber) return;
         delete throbber;
@@ -255,11 +244,6 @@ void MInstall::splashSetThrobber(bool active)
     }
     labelSplash->update();
 }
-void MInstall::splashThrob()
-{
-    ++throbPos;
-    labelSplash->update();
-}
 
 // turn auto-mount off and on
 void MInstall::setupAutoMount(bool enabled)
@@ -267,13 +251,11 @@ void MInstall::setupAutoMount(bool enabled)
     proc.log(__PRETTY_FUNCTION__, MProcess::Section);
 
     if (autoMountEnabled == enabled) return;
-    QFileInfo finfo;
     // check if the systemctl program is present
     bool have_sysctl = false;
     const QStringList &envpath = QProcessEnvironment::systemEnvironment().value("PATH").split(':');
     for (const QString &path : envpath) {
-        finfo.setFile(path + "/systemctl");
-        if (finfo.isExecutable()) {
+        if (QFileInfo(path + "/systemctl").isExecutable()) {
             have_sysctl = true;
             break;
         }
@@ -283,8 +265,7 @@ void MInstall::setupAutoMount(bool enabled)
     if (proc.shell("ps -e | grep 'udisksd'")) udisksd_running = true;
     // create a list of rules files that are being temporarily overridden
     QStringList udev_temp_mdadm_rules;
-    finfo.setFile("/run/udev");
-    if (finfo.isDir()) {
+    if (QFileInfo("/run/udev").isDir()) {
         proc.shell("grep -El '^[^#].*mdadm (-I|--incremental)' /lib/udev/rules.d/*", nullptr, true);
         udev_temp_mdadm_rules = proc.readOutLines();
         for (QString &rule : udev_temp_mdadm_rules) {
@@ -307,7 +288,7 @@ void MInstall::setupAutoMount(bool enabled)
         }
         // create temporary blank overrides for all udev rules which
         // automatically start Linux Software RAID array members
-        if (!QDir().exists("/run/udev/rules.d")) proc.mkpath("/run/udev/rules.d");
+        proc.mkpath("/run/udev/rules.d");
         for (const QString &rule : qAsConst(udev_temp_mdadm_rules)) {
             proc.exec("touch", {rule});
         }
@@ -338,22 +319,8 @@ void MInstall::setupAutoMount(bool enabled)
     autoMountEnabled = enabled;
 }
 
-/////////////////////////////////////////////////////////////////////////
-// util functions
-
-void MInstall::updateCursor(const Qt::CursorShape shape)
-{
-    if (shape != Qt::ArrowCursor) {
-        qApp->setOverrideCursor(QCursor(shape));
-    } else {
-        while (qApp->overrideCursor() != nullptr) {
-            qApp->restoreOverrideCursor();
-        }
-    }
-    qApp->processEvents();
-}
-
-bool MInstall::pretendToInstall(int space, long steps)
+// Installation simulation
+bool MInstall::pretendToInstall(int space, long steps) noexcept
 {
     proc.advance(space, steps);
     proc.status(tr("Pretending to install %1").arg(PROJECTNAME));
@@ -366,41 +333,47 @@ bool MInstall::pretendToInstall(int space, long steps)
 }
 
 // process the next phase of installation if possible
-bool MInstall::processNextPhase()
+bool MInstall::processNextPhase() noexcept
 {
     try {
         widgetStack->setEnabled(true);
-        // Phase < 0 = install has been aborted (Phase -2 on close)
-        if (proc.halted()) return false;
-        // Phase 0 = install not started yet, Phase 1 = install in progress
-        // Phase 2 = waiting for operator input, Phase 3 = post-install steps
-        if (phase == 0) { // no install started yet
+        if (proc.halted()) throw ""; // Abortion
+        if (!modeOOBE && phase == Ready) { // no install started yet
+            phase = Preparing;
             proc.advance(-1, -1);
             proc.status(tr("Preparing to install %1").arg(PROJECTNAME));
+
+            // Load defaults for configuration phase
+            bootman->buildBootLists();
+            swapman->setupDefaults();
+            manageConfig(ConfigLoadB);
+
             if (!partman->checkTargetDrivesOK()) return false;
-            phase = 1; // installation.
+            autoMountEnabled = true; // disable auto mount by force
+            if (!pretend) setupAutoMount(false);
 
             // cleanup previous mounts
             cleanup(false);
 
             // the core of the installation
+            phase = Installing;
             if (!pretend) {
                 proc.advance(11, partman->countPrepSteps());
                 partman->prepStorage();
                 base->install();
             } else {
-                if (!pretendToInstall(14, 200)) return false;
-                if (!pretendToInstall(80, 500)) return false;
+                if (!pretendToInstall(14, 100)) throw "";
+                if (!pretendToInstall(80, 100)) throw "";
             }
             if (widgetStack->currentWidget() != pageProgress) {
                 progInstall->setEnabled(false);
                 proc.status(tr("Paused for required operator input"));
                 QApplication::beep();
             }
-            phase = 2;
+            phase = WaitingForInfo;
         }
-        if (phase == 2 && widgetStack->currentWidget() == pageProgress) {
-            phase = 3;
+        if (phase == WaitingForInfo && widgetStack->currentWidget() == pageProgress) {
+            phase = Configuring;
             progInstall->setEnabled(true);
             pushBack->setEnabled(false);
             if (!pretend) {
@@ -409,18 +382,17 @@ bool MInstall::processNextPhase()
                 if (oem) oobe->enable();
                 oobe->process();
                 manageConfig(ConfigSave);
-                config->dumpDebug();
                 proc.exec("/bin/sync"); // the sync(2) system call will block the GUI
                 swapman->install();
                 bootman->install();
-                if (oem) proc.shell("sed -i 's/splash\\b/nosplash/g' /mnt/antiX/boot/grub/grub.cfg");
             } else if (!pretendToInstall(5, 100)) {
-                return false;
+                throw "";
             }
-            phase = 4;
             proc.advance(1, 1);
             proc.status(tr("Cleaning up"));
             cleanup();
+
+            phase = Finished;
             proc.status(tr("Finished"));
             if (!pretend && appArgs.isSet("reboot")) {
                 proc.shell("/usr/local/bin/persist-config --shutdown --command reboot &");
@@ -430,12 +402,45 @@ bool MInstall::processNextPhase()
             }
             gotoPage(Step::End);
         }
+        // This OOBE phase is only run under --oobe mode.
+        if (modeOOBE && phase == Ready) {
+            phase = OutOfBox;
+            labelSplash->setText(tr("Configuring sytem. Please wait."));
+            gotoPage(Step::Splash);
+
+            if (!pretend) oobe->process();
+            else if (!pretendToInstall(1, 100)) throw "";
+
+            phase = Finished;
+            labelSplash->setText(tr("Configuration complete. Restarting system."));
+            proc.exec("/usr/sbin/reboot");
+        }
     } catch (const char *msg) {
+        if (!msg || !msg[0] || abortion) {
+            msg = QT_TR_NOOP("The installation was aborted.");
+        }
         proc.log("FAILED Phase " + QString::number(phase) + " - " + msg, MProcess::Fail);
-        if (!proc.halted()) {
-            boxMain->setEnabled(false);
-            QMessageBox::critical(this, windowTitle(), tr(msg));
-            updateCursor(Qt::WaitCursor);
+        proc.setExceptionMode(nullptr);
+
+        const bool closing = (abortion == Closing);
+        labelSplash->setText(tr(msg));
+        abortUI(false, closing);
+        proc.unhalt();
+        if (!modeOOBE) {
+            manageConfig(ConfigSave);
+            cleanup();
+        }
+
+        abortion = Aborted;
+        if (closing) this->close();
+        else {
+            splashSetThrobber(false);
+            boxMain->unsetCursor();
+            // Close should be the right button at this stage.
+            disconnect(pushNext);
+            connect(pushNext, &QPushButton::clicked, this, &MInstall::close);
+            pushNext->setText(pushClose->text());
+            pushNext->show();
         }
         return false;
     }
@@ -443,7 +448,7 @@ bool MInstall::processNextPhase()
     return true;
 }
 
-void MInstall::manageConfig(enum ConfigAction mode)
+void MInstall::manageConfig(enum ConfigAction mode) noexcept
 {
     if (mode == ConfigSave) {
         delete config;
@@ -496,12 +501,15 @@ void MInstall::manageConfig(enum ConfigAction mode)
             if (advanced) bootman->manageConfig(*config);
             oobe->manageConfig(*config, mode==ConfigSave);
         }
+    } else if (mode == ConfigLoadB) {
+        oobe->manageConfig(*config, false);
     }
 
     if (mode == ConfigSave) {
         config->sync();
         QFile::remove("/etc/minstalled.conf");
         QFile::copy(config->fileName(), "/etc/minstalled.conf");
+        config->dumpDebug();
     }
 
     if (config->bad) {
@@ -511,40 +519,18 @@ void MInstall::manageConfig(enum ConfigAction mode)
     }
 }
 
-bool MInstall::saveHomeBasic()
-{
-    proc.log(__PRETTY_FUNCTION__, MProcess::Section);
-    QString homedir("/");
-    QString homedev = partman->getMountDev("/home", true);
-    if (homedev.isEmpty() || partman->willFormat("/home")) {
-        if (partman->willFormat("/")) return true;
-        homedev = partman->getMountDev("/", true);
-        homedir = "/home";
-    }
-
-    // Just in case the device or mount point is in use elsewhere.
-    proc.exec("/usr/bin/umount", {"-q", homedev});
-    proc.exec("/usr/bin/umount", {"-q", "/mnt/antiX"});
-
-    // Store a listing of /home to compare with the user name given later.
-    mkdir("/mnt/antiX", 0755);
-    bool ok = proc.exec("/bin/mount", {"-o", "ro", homedev, "/mnt/antiX"});
-    if (ok) {
-        QDir hd("/mnt/antiX" + homedir);
-        ok = hd.exists() && hd.isReadable();
-        listHomes = hd.entryList(QDir::Dirs);
-        proc.exec("/usr/bin/umount", {"-l", "/mnt/antiX"});
-    }
-    return ok;
-}
-
 // logic displaying pages
-int MInstall::showPage(int curr, int next)
+int MInstall::showPage(int curr, int next) noexcept
 {
     if (next == Step::Splash) { // Enter splash screen
+        boxMain->setCursor(Qt::WaitCursor);
         splashSetThrobber(appConf.value("SPLASH_THROBBER", true).toBool());
     } else if (curr == Step::Splash) { // Leave splash screen
+        labelSplash->clear();
         splashSetThrobber(false);
+        boxMain->unsetCursor();
+    } else if (curr == Step::Terms && next > curr) {
+        if (modeOOBE) return Step::Network;
     } else if (curr == Step::Disk && next > curr) {
         if (radioEntireDisk->isChecked()) {
             if (!automatic) {
@@ -572,14 +558,14 @@ int MInstall::showPage(int curr, int next)
             manageConfig(ConfigLoadB);
             checkHibernation->setChecked(checkHibernationReg->isChecked());
             swapman->setupDefaults();
-            return Step::Network;
+            return oem ? Step::Progress : Step::Network;
         }
     } else if (curr == Step::Partitions && next > curr) {
         if (!partman->composeValidate(automatic, PROJECTNAME)) {
             nextFocus = treePartitions;
             return curr;
         }
-        if (!pretend && !saveHomeBasic()) {
+        if (!pretend && !(base && base->saveHomeBasic())) {
             QMessageBox::critical(this, windowTitle(),
                 tr("The data in /home cannot be preserved because"
                     " the required information could not be obtained."));
@@ -587,34 +573,34 @@ int MInstall::showPage(int curr, int next)
         }
         return Step::Boot;
     } else if (curr == Step::Boot && next > curr) {
-        if (oem) return Step::Progress;
-        return Step::Services + 1;
-    } else if (curr == Step::UserAccounts && next > curr) {
-        nextFocus = oobe->validateUserInfo(automatic);
-        if (nextFocus) return curr;
-        // Check for pre-existing /home directory, see if user directory already exists.
-        haveOldHome = listHomes.contains(textUserName->text());
-        if (!haveOldHome) return Step::Progress; // Skip pageOldHome
-        else {
-            const QString &str = tr("The home directory for %1 already exists.");
-            labelOldHome->setText(str.arg(textUserName->text()));
-        }
+        return oem ? Step::Progress : Step::Network;
     } else if (curr == Step::Network && next > curr) {
         nextFocus = oobe->validateComputerName();
         if (nextFocus) return curr;
     } else if (curr == Step::Network && next < curr) { // Backward
-        return Step::Boot; // Skip pageServices
+        if (modeOOBE) return Step::Terms;
+        else return Step::Boot; // Skip pageServices
     } else if (curr == Step::Localization && next > curr) {
         if (!pretend && oobe->haveSnapshotUserAccounts) {
             return Step::Progress; // Skip pageUserAccounts and pageOldHome
         }
+    } else if (curr == Step::UserAccounts && next > curr) {
+        nextFocus = oobe->validateUserInfo(automatic);
+        if (nextFocus) return curr;
+        // Check for pre-existing /home directory, see if user directory already exists.
+        haveOldHome = base && base->homes.contains(textUserName->text());
+        if (!haveOldHome) return Step::Progress; // Skip pageOldHome
+        else {
+            const QString &str = tr("The home directory for %1 already exists.");
+            labelOldHome->setText(str.arg(textUserName->text()));
+        }
     } else if (curr == Step::OldHome && next < curr) { // Backward
         if (!pretend && oobe->haveSnapshotUserAccounts) {
             return Step::Localization; // Skip pageUserAccounts and pageOldHome
         }
     } else if (curr == Step::Progress && next < curr) { // Backward
         if (oem) return Step::Boot;
-        if (!haveOldHome) {
+        else if (!haveOldHome) {
             // skip pageOldHome
             if (!pretend && oobe->haveSnapshotUserAccounts) {
                 return Step::Localization;
@@ -628,7 +614,7 @@ int MInstall::showPage(int curr, int next)
     return next;
 }
 
-void MInstall::pageDisplayed(int next)
+void MInstall::pageDisplayed(int next) noexcept
 {
     bool enableBack = true, enableNext = true;
     if (!modeOOBE) {
@@ -642,8 +628,8 @@ void MInstall::pageDisplayed(int next)
     switch (next) {
     case Step::Terms:
         textHelp->setText("<p><b>" + tr("General Instructions") + "</b><br/>"
-            + tr("BEFORE PROCEEDING, CLOSE ALL OTHER APPLICATIONS.") + "</p>"
-            "<p>" + tr("On each page, please read the instructions, make your selections, and then click on Next when you are ready to proceed."
+            + (modeOOBE ? "" : (tr("BEFORE PROCEEDING, CLOSE ALL OTHER APPLICATIONS.") + "</p><p>"))
+            + tr("On each page, please read the instructions, make your selections, and then click on Next when you are ready to proceed."
                 " You will be prompted for confirmation before any destructive actions are performed.") + "</p>"
             + "<p><b>" + tr("Limitations") + "</b><br/>"
             + tr("Remember, this software is provided AS-IS with no warranty what-so-ever."
@@ -669,14 +655,7 @@ void MInstall::pageDisplayed(int next)
             + tr("If you need more control over where %1 is installed to, select \"<b>%2</b>\" and click <b>Next</b>."
                 " On the next page, you will then be able to select and configure the storage devices and"
                 " partitions you need.").arg(PROJECTNAME, radioCustomPart->text().remove('&')) + "</p>");
-        if (proc.halted()) {
-            updateCursor(Qt::WaitCursor);
-            phase = 0;
-            proc.unhalt();
-            updateCursor();
-        }
-        enableNext = radioCustomPart->isChecked()
-            || !boxEncryptAuto->isChecked() || textCryptoPass->isValid();
+        enableNext = radioCustomPart->isChecked() || !boxEncryptAuto->isChecked() || autopart->passCrypto.isValid();
         break;
 
     case Step::Partitions:
@@ -759,7 +738,7 @@ void MInstall::pageDisplayed(int next)
             "<p><b>" + tr("Virtual Devices") + "</b><br/>"
             + tr("If the intaller detects any virtual devices such as opened LUKS partitions, LVM logical volumes or software-based RAID volumes, they may be used for the installation.") + "</p>"
             "<p>" + tr("The use of virtual devices (beyond preserving encrypted file systems) is an advanced feature. You may have to edit some files (eg. initramfs, crypttab, fstab) to ensure the virtual devices used are created upon boot.") + "</p>");
-        enableNext = !(boxCryptoPass->isEnabledTo(boxCryptoPass->parentWidget())) || textCryptoPassCust->isValid();
+        enableNext = !(boxCryptoPass->isEnabledTo(boxCryptoPass->parentWidget())) || passCryptoCust->isValid();
         break;
 
     case Step::Boot: // Start of installation.
@@ -774,11 +753,6 @@ void MInstall::pageDisplayed(int next)
             + tr("Setting the size to 0 has the same effect as unchecking this option.") + "</p>");
 
         enableBack = false;
-        if (phase <= 0) {
-            bootman->buildBootLists();
-            swapman->setupDefaults();
-            manageConfig(ConfigLoadB);
-        }
         break;
 
     case Step::Services:
@@ -791,7 +765,7 @@ void MInstall::pageDisplayed(int next)
                              "<p>The computer and domain names can contain only alphanumeric characters, dots, hyphens. They cannot contain blank spaces, start or end with hyphens</p>"
                              "<p>The SaMBa Server needs to be activated if you want to use it to share some of your directories or printer "
                              "with a local computer that is running MS-Windows or Mac OSX.</p>"));
-        if (modeOOBE) enableBack = false;
+        if (modeOOBE) enableBack = true;
         else enableBack = radioCustomPart->isChecked();
         break;
 
@@ -825,6 +799,7 @@ void MInstall::pageDisplayed(int next)
             " does not need to be secure, such as a public terminal.") + "</p>");
         if (!nextFocus) nextFocus = textUserName;
         oobe->userPassValidationChanged();
+        enableNext = pushNext->isEnabled();
         break;
 
     case Step::OldHome:
@@ -867,6 +842,7 @@ void MInstall::pageDisplayed(int next)
             + "</p><p>"
             + tr("Complete these steps at your own pace. The installer will wait for your input if necessary.")
             + "</p>");
+        enableBack = !oem || radioCustomPart->isChecked();
         enableNext = false;
         break;
 
@@ -894,7 +870,7 @@ void MInstall::pageDisplayed(int next)
     pushNext->setEnabled(enableNext);
 }
 
-void MInstall::gotoPage(int next)
+void MInstall::gotoPage(int next) noexcept
 {
     pushBack->setEnabled(false);
     pushNext->setEnabled(false);
@@ -922,7 +898,7 @@ void MInstall::gotoPage(int next)
     pushNext->setIconSize(isize);
     if (next > Step::End) {
         // finished
-        updateCursor(Qt::WaitCursor);
+        qApp->setOverrideCursor(Qt::WaitCursor);
         if (!pretend && checkExitReboot->isChecked()) {
             proc.shell("/usr/local/bin/persist-config --shutdown --command reboot &");
         }
@@ -944,7 +920,7 @@ void MInstall::gotoPage(int next)
     // automatic installation
     if (automatic) {
         if (!MSettings::isBadWidget(widgetStack->currentWidget()) && next > curr) {
-            pushNext->click();
+            QTimer::singleShot(0, pushNext, &QPushButton::click);
         } else if (curr!=0) { // failed validation
             automatic = false;
         }
@@ -953,34 +929,14 @@ void MInstall::gotoPage(int next)
     // process next installation phase
     if (next == Step::Boot || next == Step::Progress
         || (radioEntireDisk->isChecked() && next == Step::Network)) {
-        if (modeOOBE) {
-            updateCursor(Qt::BusyCursor);
-            labelSplash->setText(tr("Configuring sytem. Please wait."));
-            gotoPage(Step::Splash);
-            try {
-                oobe->process();
-                labelSplash->setText(tr("Configuration complete. Restarting system."));
-                proc.exec("/usr/sbin/reboot");
-                qApp->exit(EXIT_SUCCESS);
-            } catch (const char *msg) {
-                splashSetThrobber(false);
-                proc.log(QString("FAILED OOBE - ") + msg, MProcess::Fail);
-                labelSplash->setText(tr(msg));
-                pushClose->show();
-            }
-        } else if (!processNextPhase() && phase > -2) {
-            cleanup(false);
-            gotoPage(Step::Disk);
-            boxMain->setEnabled(true);
-        }
-        updateCursor();
+        processNextPhase();
     }
 }
 
 /////////////////////////////////////////////////////////////////////////
 // event handlers
 
-bool MInstall::eventFilter(QObject *watched, QEvent *event)
+bool MInstall::eventFilter(QObject *watched, QEvent *event) noexcept
 {
     if (event->type() != QEvent::Paint) return false;
     else if (watched == labelSplash) {
@@ -990,26 +946,24 @@ bool MInstall::eventFilter(QObject *watched, QEvent *event)
         const int lW = labelSplash->width(), lH = labelSplash->height();
         painter.translate(lW / 2, lH / 2);
         painter.scale(lW / 200.0, lH / 200.0);
-        QColor color = labelSplash->palette().text().color();
-        color.setRed(255 - color.red());
-        color.setAlpha(70);
-        QPen pen(color.darker());
+        // Draw the load indicator on the splash screen.
+        const int blades = 12;
+        const qreal angle = 360.0 / blades;
+        painter.rotate(angle * throbPos);
+        float hue = 1.0, alpha = 0.18;
+        const float revs = throbPos / blades;
+        const float huestep = ((120.0 + (revs<240 ? revs : 240))/360.0) / blades;
+        const float alphastep = 0.18 / blades;
+        QPen pen;
         pen.setWidth(3);
         pen.setJoinStyle(Qt::MiterJoin);
-        painter.setPen(pen);
-        // Draw the load indicator on the splash screen.
-        const int count = 16;
-        const int alphaMin = 0, alphaMax = 70;
-        const QPoint blade[] = {
-            QPoint(0, -6), QPoint(9, -75),
-            QPoint(0, -93), QPoint(-9, -75)
-        };
-        const qreal angle = 360.0 / count;
-        painter.rotate(angle * throbPos);
-        const int astep = (alphaMax - alphaMin) / count;
-        for (int ixi = alphaMin; ixi <= alphaMax; ixi += astep) {
-            color.setAlpha(ixi);
+        for (int ixi=0; ixi<blades; ++ixi) {
+            const QColor &color = QColor::fromHsvF(hue, 1.0, 1.0, alpha);
+            hue -= huestep, alpha += alphastep;
             painter.setBrush(color);
+            pen.setColor(color.darker());
+            painter.setPen(pen);
+            const QPoint blade[] = {{-15, -6}, {15, -75}, {0, -93}, {-15, -75}};
             painter.drawConvexPolygon(blade, 4);
             painter.rotate(angle);
         }
@@ -1022,7 +976,7 @@ bool MInstall::eventFilter(QObject *watched, QEvent *event)
     return false;
 }
 
-void MInstall::changeEvent(QEvent *event)
+void MInstall::changeEvent(QEvent *event) noexcept
 {
     const QEvent::Type etype = event->type();
     if (etype == QEvent::ApplicationPaletteChange || etype == QEvent::PaletteChange || etype == QEvent::StyleChange) {
@@ -1035,81 +989,75 @@ void MInstall::changeEvent(QEvent *event)
     }
 }
 
-void MInstall::closeEvent(QCloseEvent *event)
+void MInstall::closeEvent(QCloseEvent *event) noexcept
 {
-    if (abortUI()) {
-        event->accept();
-        phase = -2;
-        if (!modeOOBE) cleanup();
-        else if (!pretend) {
-            proc.unhalt();
-            proc.exec("/usr/sbin/shutdown", {"-hP", "now"});
-        }
-        QWidget::closeEvent(event);
-        if (widgetStack->currentWidget() != pageEnd) {
-            qApp->exit(EXIT_FAILURE);
-        } else {
-            proc.waitForFinished();
-            qApp->exit(EXIT_SUCCESS);
-        }
-    } else {
+    if (phase > Ready && phase < Finished && abortion != Aborted) {
+        // Currently installing, could be pending abortion (but not finished aborting).
         event->ignore();
+        abortUI(true, true);
+    } else if (modeOOBE) {
+        // Shutdown for pending or fully aborted OOBE
+        event->ignore();
+        labelSplash->clear();
+        gotoPage(Step::Splash);
+        proc.unhalt();
+        proc.exec("/usr/sbin/shutdown", {"-hP", "now"});
+    } else {
+        // Fully aborted installation (but not OOBE).
+        event->accept();
+        if (phase == StartingUp) proc.halt(true);
+        if (checkmd5) checkmd5->halt(true);
     }
 }
 
 // Override QDialog::reject() so Escape won't close the window.
-void MInstall::reject()
+void MInstall::reject() noexcept
 {
+    if (checkmd5) checkmd5->halt();
 }
 
 /////////////////////////////////////////////////////////////////////////
 // slots
 
-void MInstall::on_pushNext_clicked()
+void MInstall::on_pushNext_clicked() noexcept
 {
     gotoPage(widgetStack->currentIndex() + 1);
 }
-void MInstall::on_pushBack_clicked()
+void MInstall::on_pushBack_clicked() noexcept
 {
     gotoPage(widgetStack->currentIndex() - 1);
 }
 
-void MInstall::on_pushAbort_clicked()
+void MInstall::on_pushAbort_clicked() noexcept
 {
-    if(abortUI()) {
-        if (!modeOOBE) cleanup();
-        phase = -1;
-        gotoPage(Step::Terms);
-    }
-    QApplication::beep();
-    boxMain->setEnabled(true);
-    updateCursor();
+    abortUI(true, false);
 }
 
 // clicking advanced button to go to Services page
-void MInstall::on_pushServices_clicked()
+void MInstall::on_pushServices_clicked() noexcept
 {
     gotoPage(Step::Services);
 }
 
-bool MInstall::abortUI()
+void MInstall::abortUI(bool manual, bool closing) noexcept
 {
-    proc.log(__PRETTY_FUNCTION__, MProcess::Section);
-    boxMain->setEnabled(false);
     // ask for confirmation when installing (except for some steps that don't need confirmation)
-    if (phase > 0 && phase < 4) {
-        const QMessageBox::StandardButton rc = QMessageBox::warning(this, QString(),
-            tr("The installation and configuration is incomplete.\nDo you really want to stop now?"),
-            QMessageBox::Yes | QMessageBox::No, QMessageBox::No);
-        if (rc == QMessageBox::No) {
-            boxMain->setEnabled(true);
-            return false;
+    if (abortion != NoAbort) return; // Don't abort an abortion.
+    else if (phase > Ready && phase < Finished) {
+        if (manual) {
+            const QMessageBox::StandardButton rc = QMessageBox::warning(this, QString(),
+                tr("The installation and configuration is incomplete.\nDo you really want to stop now?"),
+                QMessageBox::Yes | QMessageBox::No, QMessageBox::No);
+            if (rc == QMessageBox::No) return;
+            proc.log("MANUALLY ABORTED", MProcess::Fail);
         }
     }
-    updateCursor(Qt::WaitCursor);
-    proc.log("Manual abort", MProcess::Fail);
-    proc.halt();
-    return true;
+    // At this point the abortion has not been cancelled.
+    abortion = closing ? Closing : Aborting;
+    gotoPage(Step::Splash);
+    proc.halt(true);
+    // Early phase bump if waiting on input to trigger abortion cleanup.
+    if (manual && phase == WaitingForInfo) processNextPhase();
 }
 
 // run before closing the app, do some cleanup
@@ -1119,8 +1067,6 @@ void MInstall::cleanup(bool endclean)
     if (pretend) return;
 
     if (endclean) {
-        proc.unhalt();
-        setupAutoMount(true);
         proc.exec("/usr/bin/cp", {"/var/log/minstall.log", "/mnt/antiX/var/log"});
         proc.exec("/usr/bin/rm", {"-rf", "/mnt/antiX/mnt/antiX"});
     }
@@ -1129,10 +1075,13 @@ void MInstall::cleanup(bool endclean)
     if (proc.exec("/usr/bin/mountpoint", {"-q", "/mnt/antiX/sys"}))      proc.exec("/usr/bin/umount", {"-lq", "/mnt/antiX/sys"});
     if (proc.exec("/usr/bin/mountpoint", {"-q", "/mnt/antiX/dev/shm"}))  proc.exec("/usr/bin/umount", {"-lq", "/mnt/antiX/dev/shm"});
     if (proc.exec("/usr/bin/mountpoint", {"-q", "/mnt/antiX/dev"}))      proc.exec("/usr/bin/umount", {"-lq", "/mnt/antiX/dev"});
-    if (endclean && !mountkeep) partman->unmount();
+    if (endclean) {
+        if (!mountkeep) partman->unmount();
+        setupAutoMount(true);
+    }
 }
 
-void MInstall::on_progInstall_valueChanged(int value)
+void MInstall::on_progInstall_valueChanged(int value) noexcept
 {
     if (ixTipStart < 0 || widgetStack->currentWidget() != pageProgress) {
         return; // no point displaying a new hint if it will be invisible
@@ -1192,7 +1141,7 @@ void MInstall::on_progInstall_valueChanged(int value)
     }
 }
 
-void MInstall::setupkeyboardbutton()
+void MInstall::setupkeyboardbutton() noexcept
 {
     QFile file("/etc/default/keyboard");
     if (!file.open(QFile::ReadOnly | QFile::Text)) return;
@@ -1212,20 +1161,20 @@ void MInstall::setupkeyboardbutton()
     file.close();
 }
 
-void MInstall::on_pushSetKeyboard_clicked()
+void MInstall::on_pushSetKeyboard_clicked() noexcept
 {
-    hide();
+    this->setEnabled(false);
     if (proc.shell("command -v  system-keyboard-qt >/dev/null 2>&1")) {
         proc.exec("system-keyboard-qt");
     } else {
         proc.shell("env GTK_THEME='Adwaita' fskbsetting");
     }
-    show();
     setupkeyboardbutton();
+    this->setEnabled(true);
 }
 
-void MInstall::on_radioEntireDisk_toggled(bool checked)
+void MInstall::on_radioEntireDisk_toggled(bool checked) noexcept
 {
     boxAutoPart->setEnabled(checked);
-    pushNext->setEnabled(!checked || !boxEncryptAuto->isChecked() || textCryptoPass->isValid());
+    pushNext->setEnabled(!checked || !boxEncryptAuto->isChecked() || autopart->passCrypto.isValid());
 }
