@@ -1,6 +1,5 @@
 /***************************************************************************
  * SafeCache Class - for temporarily caching sensitive files
- ***************************************************************************
  *
  *   Copyright (C) 2019 by AK-47
  *   Licensed under the Apache License, Version 2.0 (the "License");
@@ -22,7 +21,7 @@
 #include <fcntl.h>
 #include "safecache.h"
 
-SafeCache::SafeCache()
+SafeCache::SafeCache() noexcept
 {
     reserve(16384);
 }
@@ -33,7 +32,7 @@ SafeCache::~SafeCache()
 }
 
 // to completely free the key use parameters nullptr, 0
-bool SafeCache::load(const char *filename, int length)
+bool SafeCache::load(const char *filename, int length) noexcept
 {
     bool ok = false;
     erase();
@@ -66,7 +65,7 @@ bool SafeCache::load(const char *filename, int length)
     return ok;
 }
 
-bool SafeCache::save(const char *filename, mode_t mode)
+bool SafeCache::save(const char *filename, mode_t mode) noexcept
 {
     bool ok = false;
     int fd = open(filename, O_CREAT | O_TRUNC | O_WRONLY, mode);
@@ -80,7 +79,7 @@ bool SafeCache::save(const char *filename, mode_t mode)
     return ok;
 }
 
-void SafeCache::erase()
+void SafeCache::erase() noexcept
 {
     fill(0xAA);
     fill(0x55);
